@@ -26,6 +26,8 @@
 #include <unistd.h>
 #include <fcntl.h>
 
+#include <vicious.h>
+
 #include "gdm.h"
 #include "gdm-net.h"
 #include "server.h"
@@ -169,7 +171,7 @@ gdm_display_manage (GdmDisplay *d)
     /* If we have an old slave process hanging around, kill it */
     if (d->slavepid) {
 	    if (kill (d->slavepid, SIGINT) == 0)
-		    waitpid (d->slavepid, 0, 0);
+		    ve_waitpid_no_signal (d->slavepid, 0, 0);
 	    d->slavepid = 0;
     }
 
@@ -185,6 +187,8 @@ gdm_display_manage (GdmDisplay *d)
     case 0:
 	setpgid (0, 0);
 
+	d->slavepid = getpid ();
+
 	/* Close XDMCP fd in slave process */
 	if (GdmXdmcp)
 	    gdm_xdmcp_close ();
@@ -201,9 +205,9 @@ gdm_display_manage (GdmDisplay *d)
 
 	/* No error checking here - if it's messed the best response
          * is to ignore & try to continue */
-	open("/dev/null", O_RDONLY); /* open stdin - fd 0 */
-	open("/dev/null", O_RDWR); /* open stdout - fd 1 */
-	open("/dev/null", O_RDWR); /* open stderr - fd 2 */
+	gdm_open_dev_null (O_RDONLY); /* open stdin - fd 0 */
+	gdm_open_dev_null (O_RDWR); /* open stdout - fd 1 */
+	gdm_open_dev_null (O_RDWR); /* open stderr - fd 2 */
 
 	openlog ("gdm", LOG_PID, LOG_DAEMON);
 
@@ -277,7 +281,7 @@ gdm_display_unmanage (GdmDisplay *d)
     /* Kill slave */
     if (d->slavepid != 0) {
 	if (kill (d->slavepid, SIGTERM) == 0)
-		waitpid (d->slavepid, 0, 0);
+		ve_waitpid_no_signal (d->slavepid, 0, 0);
 	d->slavepid = 0;
     }
     
@@ -345,6 +349,11 @@ gdm_display_dispose (GdmDisplay *d)
     g_free (d->authfile_gdm);
     d->authfile_gdm = NULL;
 
+    if (d->xnest_temp_auth_file != NULL)
+	    unlink (d->xnest_temp_auth_file);
+    g_free (d->xnest_temp_auth_file);
+    d->xnest_temp_auth_file = NULL;
+
     if (d->auths) {
 	GSList *tmpauth = d->auths;
 
