@@ -112,6 +112,14 @@ gdm_server_reinit (GdmDisplay *disp)
 	gdm_debug ("gdm_server_reinit: Server for %s is about to be reinitialized!", disp->name);
 
 	kill (disp->servpid, SIGHUP);
+
+	/* HACK! the Xserver can't really tell us when it got the hup signal,
+	 * so we are really stuck just going to sleep for a bit hoping that
+	 * the kernel will tell the X server and that will run, else we will
+	 * get whacked ourselves after we open the connection and we'll think
+	 * it's an X screwup, which is really OK to happen and will just
+	 * restart the Xserver, it's just more nasty.  Oh how fun */
+	sleep (1);
 }
 
 /**
@@ -162,6 +170,8 @@ gdm_server_stop (GdmDisplay *disp)
     }
 
     gdm_server_wipe_cookies (disp);
+
+    gdm_slave_whack_temp_auth_file ();
 }
 
 static gboolean
@@ -761,9 +771,9 @@ gdm_server_spawn (GdmDisplay *d)
 
 	/* No error checking here - if it's messed the best response
          * is to ignore & try to continue */
-	open ("/dev/null", O_RDONLY); /* open stdin - fd 0 */
-	open ("/dev/null", O_RDWR); /* open stdout - fd 1 */
-	open ("/dev/null", O_RDWR); /* open stderr - fd 2 */
+	gdm_open_dev_null (O_RDONLY); /* open stdin - fd 0 */
+	gdm_open_dev_null (O_RDWR); /* open stdout - fd 1 */
+	gdm_open_dev_null (O_RDWR); /* open stderr - fd 2 */
 
 	openlog ("gdm", LOG_PID, LOG_DAEMON);
 
