@@ -40,6 +40,7 @@
 #include <netinet/in.h>
 #include <arpa/inet.h>
 #include <X11/Xlib.h>
+#include <X11/Xatom.h>
 #ifdef HAVE_LIBXINERAMA
 #include <X11/extensions/Xinerama.h>
 #endif
@@ -67,6 +68,7 @@ static GdmDisplay *d;
 static gchar *login = NULL;
 static sigset_t mask;
 static gboolean greet = FALSE;
+static gboolean configurator = FALSE;
 static gboolean remanage_asap = FALSE;
 static gboolean do_timed_login = FALSE; /* if this is true,
 					   login the timed login */
@@ -151,10 +153,12 @@ static void     gdm_slave_exit (gint status, const gchar *format, ...) G_GNUC_PR
 static void     gdm_child_exit (gint status, const gchar *format, ...) G_GNUC_PRINTF (2, 3);
 static gint     gdm_slave_exec_script (GdmDisplay *d, const gchar *dir,
 				       const char *login, struct passwd *pwent,
-				       gboolean pass_stdout);
+				       gboolean pass_stdout, 
+				       gboolean set_parent);
 static gchar *  gdm_parse_enriched_login (const gchar *s, GdmDisplay *display);
 static void	gdm_slave_handle_notify (const char *msg);
-
+static void	create_temp_auth_file (void);
+static void	set_xnest_parent_stuff (void);
 
 /* Yay thread unsafety */
 static gboolean x_error_occured = FALSE;
@@ -185,6 +189,38 @@ check_notifies_now (void)
 	g_list_free (list);
 }
 
+static void
+gdm_slave_desensitize_config (void)
+{
+	if (configurator &&
+	    d->dsp != NULL) {
+		gulong foo = 1;
+		Atom atom = XInternAtom (d->dsp,
+					 "_GDM_SETUP_INSENSITIVE",
+					 False);
+		XChangeProperty (d->dsp,
+				 DefaultRootWindow (d->dsp),
+				 atom,
+				 XA_CARDINAL, 32, PropModeReplace,
+				 (unsigned char *) &foo, 1);
+		XSync (d->dsp, False);
+	}
+
+}
+
+static void
+gdm_slave_sensitize_config (void)
+{
+	if (d->dsp != NULL) {
+		XDeleteProperty (d->dsp,
+				 DefaultRootWindow (d->dsp),
+				 XInternAtom (d->dsp,
+					      "_GDM_SETUP_INSENSITIVE",
+					      False));
+		XSync (d->dsp, False);
+	}
+}
+
 /* ignore handlers */
 static int
 ignore_xerror_handler (Display *disp, XErrorEvent *evt)
@@ -336,7 +372,8 @@ setup_automatic_session (GdmDisplay *display, const char *name)
 	/* Run the init script. gdmslave suspends until script
 	 * has terminated */
 	gdm_slave_exec_script (display, GdmDisplayInit, NULL, NULL,
-			       FALSE /* pass_stdout */);
+			       FALSE /* pass_stdout */,
+			       TRUE /* set_parent */);
 
 	gdm_debug ("setup_automatic_session: DisplayInit script finished");
 
@@ -429,6 +466,8 @@ gdm_slave_whack_greeter (void)
 
 	gdm_slave_send_num (GDM_SOP_GREETPID, 0);
 
+	gdm_slave_whack_temp_auth_file ();
+
 	gdm_sigchld_block_pop ();
 }
 
@@ -712,9 +751,9 @@ focus_first_x_window (const char *class_res_name)
 
 	/* No error checking here - if it's messed the best response
          * is to ignore & try to continue */
-	open ("/dev/null", O_RDONLY); /* open stdin - fd 0 */
-	open ("/dev/null", O_RDWR); /* open stdout - fd 1 */
-	open ("/dev/null", O_RDWR); /* open stderr - fd 2 */
+	gdm_open_dev_null (O_RDONLY); /* open stdin - fd 0 */
+	gdm_open_dev_null (O_RDWR); /* open stdout - fd 1 */
+	gdm_open_dev_null (O_RDWR); /* open stderr - fd 2 */
 
 	openlog ("gdm", LOG_PID, LOG_DAEMON);
 
@@ -835,9 +874,9 @@ run_config (GdmDisplay *display, struct passwd *pwent)
 
 		/* No error checking here - if it's messed the best response
 		 * is to ignore & try to continue */
-		open ("/dev/null", O_RDONLY); /* open stdin - fd 0 */
-		open ("/dev/null", O_RDWR); /* open stdout - fd 1 */
-		open ("/dev/null", O_RDWR); /* open stderr - fd 2 */
+		gdm_open_dev_null (O_RDONLY); /* open stdin - fd 0 */
+		gdm_open_dev_null (O_RDWR); /* open stdout - fd 1 */
+		gdm_open_dev_null (O_RDWR); /* open stderr - fd 2 */
 
 		openlog ("gdm", LOG_PID, LOG_DAEMON);
 
@@ -873,6 +912,7 @@ run_config (GdmDisplay *display, struct passwd *pwent)
 
 		_exit (0);
 	} else {
+		configurator = TRUE;
 		/* XXX: is this a race if we don't push a sigchld block?
 		 * but then we don't get a signal for restarting the greeter */
 		/* wait for the config proggie to die */
@@ -882,12 +922,15 @@ run_config (GdmDisplay *display, struct passwd *pwent)
 			 * handler */
 			ve_waitpid_no_signal (pid, 0, 0);
 		display->sesspid = 0;
+		configurator = FALSE;
 	}
 }
 
 static void
 restart_the_greeter (void)
 {
+	gdm_slave_desensitize_config ();
+
 	/* no login */
 	g_free (login);
 	login = NULL;
@@ -919,6 +962,8 @@ restart_the_greeter (void)
 
 	if (greeter_no_focus)
 		gdm_slave_greeter_ctl_no_ret (GDM_NOFOCUS, "");
+
+	gdm_slave_sensitize_config ();
 }
 
 static void
@@ -1329,6 +1374,53 @@ run_pictures (void)
 	g_free (response);
 }
 
+/* hakish, copy file (owned by fromuid) to a temp file owned by touid */
+static char *
+copy_auth_file (uid_t fromuid, uid_t touid, const char *file)
+{
+	uid_t old = geteuid ();
+	char *name;
+	int authfd;
+	int fromfd;
+	int bytes;
+	char buf[2048];
+
+	seteuid (fromuid);
+
+	fromfd = open (file, O_RDONLY);
+
+	if (fromfd < 0) {
+		seteuid (old);
+		return NULL;
+	}
+
+	seteuid (0);
+
+	name = g_strconcat (GdmServAuthDir, "/", d->name, ".XnestAuth", NULL);
+
+	authfd = open (name, O_TRUNC|O_WRONLY|O_CREAT, 0600);
+
+	if (authfd < 0) {
+		seteuid (old);
+		g_free (name);
+		return NULL;
+	}
+
+	/* Make it owned by the user that Xnest is started as */
+	fchown (authfd, touid, -1);
+
+	while ((bytes = read (fromfd, buf, sizeof (buf))) > 0) {
+		write (authfd, buf, bytes);
+	}
+
+	close (fromfd);
+	close (authfd);
+
+	seteuid (old);
+
+	return name;
+}
+
 static void
 gdm_slave_greeter (void)
 {
@@ -1341,12 +1433,16 @@ gdm_slave_greeter (void)
     
     /* Run the init script. gdmslave suspends until script has terminated */
     gdm_slave_exec_script (d, GdmDisplayInit, NULL, NULL,
-			   FALSE /* pass_stdout */);
+			   FALSE /* pass_stdout */,
+			   TRUE /* set_parent */);
 
     /* Open a pipe for greeter communications */
     if (pipe (pipe1) < 0 || pipe (pipe2) < 0) 
 	gdm_slave_exit (DISPLAY_ABORT, _("%s: Can't init pipe to gdmgreeter"),
 			"gdm_slave_greeter");
+
+    /* hackish ain't it */
+    create_temp_auth_file ();
     
     /* Fork. Parent is gdmslave, child is greeter process. */
     gdm_sigchld_block_push ();
@@ -1376,7 +1472,7 @@ gdm_slave_greeter (void)
 
 	gdm_close_all_descriptors (2 /* from */, -1 /* except */);
 
-	open ("/dev/null", O_RDWR); /* open stderr - fd 2 */
+	gdm_open_dev_null (O_RDWR); /* open stderr - fd 2 */
 
 	openlog ("gdm", LOG_PID, LOG_DAEMON);
 	
@@ -1402,6 +1498,9 @@ gdm_slave_greeter (void)
 	gnome_setenv ("XAUTHORITY", GDM_AUTHFILE (d), TRUE);
 	gnome_setenv ("DISPLAY", d->name, TRUE);
 
+	/* hackish ain't it */
+	set_xnest_parent_stuff ();
+
 	gnome_setenv ("LOGNAME", GdmUser, TRUE);
 	gnome_setenv ("USER", GdmUser, TRUE);
 	gnome_setenv ("USERNAME", GdmUser, TRUE);
@@ -1557,6 +1656,7 @@ gdm_slave_send (const char *str, gboolean wait_for_ack)
 	int fd;
 	char *fifopath;
 	int i;
+	uid_t old;
 
 	if (gdm_in_signal == 0)
 		gdm_debug ("Sending %s", str);
@@ -1565,7 +1665,12 @@ gdm_slave_send (const char *str, gboolean wait_for_ack)
 		gdm_got_ack = FALSE;
 
 	fifopath = g_strconcat (GdmServAuthDir, "/.gdmfifo", NULL);
+	old = geteuid ();
+	if (old != 0)
+		seteuid (0);
 	fd = open (fifopath, O_WRONLY);
+	if (old != 0)
+		seteuid (old);
 	g_free (fifopath);
 
 	/* eek */
@@ -1640,6 +1745,7 @@ send_chosen_host (GdmDisplay *disp, const char *hostname)
 	int fd;
 	char *fifopath;
 	struct hostent *host;
+	uid_t old;
 
 	host = gethostbyname (hostname);
 
@@ -1653,7 +1759,12 @@ send_chosen_host (GdmDisplay *disp, const char *hostname)
 
 	fifopath = g_strconcat (GdmServAuthDir, "/.gdmfifo", NULL);
 
+	old = geteuid ();
+	if (old != 0)
+		seteuid (0);
 	fd = open (fifopath, O_WRONLY);
+	if (old != 0)
+		seteuid (old);
 
 	g_free (fifopath);
 
@@ -1689,7 +1800,8 @@ gdm_slave_chooser (void)
 
 	/* Run the init script. gdmslave suspends until script has terminated */
 	gdm_slave_exec_script (d, GdmDisplayInit, NULL, NULL,
-			       FALSE /* pass_stdout */);
+			       FALSE /* pass_stdout */,
+			       TRUE /* set_parent */);
 
 	/* Fork. Parent is gdmslave, child is greeter process. */
 	gdm_sigchld_block_push ();
@@ -1718,8 +1830,8 @@ gdm_slave_chooser (void)
 		close (0);
 		gdm_close_all_descriptors (2 /* from */, -1 /* except */);
 
-		open ("/dev/null", O_RDONLY); /* open stdin - fd 0 */
-		open ("/dev/null", O_RDWR); /* open stderr - fd 2 */
+		gdm_open_dev_null (O_RDONLY); /* open stdin - fd 0 */
+		gdm_open_dev_null (O_RDWR); /* open stderr - fd 2 */
 
 		openlog ("gdm", LOG_PID, LOG_DAEMON);
 
@@ -2116,8 +2228,19 @@ session_child_run (struct passwd *pwent,
 
 	disp = XOpenDisplay (d->name);
 	if (disp != NULL) {
+		Cursor xcursor;
+			
 		XSetInputFocus (disp, PointerRoot,
 				RevertToPointerRoot, CurrentTime);
+
+		/* return left pointer */
+		xcursor = XCreateFontCursor (disp, GDK_LEFT_PTR);
+		XDefineCursor (disp,
+			       DefaultRootWindow (disp),
+			       xcursor);
+		XFreeCursor (disp, xcursor);
+		XSync (disp, False);
+
 		XCloseDisplay (disp);
 	}
 
@@ -2135,8 +2258,14 @@ session_child_run (struct passwd *pwent,
 	 * unless in failsafe mode which needs to work when there is
 	 * no diskspace as well */
 	if ( ! failsafe && home_dir_ok) {
+		uid_t old = geteuid ();
+		uid_t oldg = getegid ();
+		setegid (pwent->pw_gid);
+		seteuid (pwent->pw_uid);
 		logfd = open (g_strconcat (home_dir, "/.xsession-errors", NULL),
 			      O_CREAT|O_TRUNC|O_WRONLY, 0644);
+		seteuid (old);
+		setegid (oldg);
 		if (logfd != -1) {
 			dup2 (logfd, 1);
 			dup2 (logfd, 2);
@@ -2144,25 +2273,26 @@ session_child_run (struct passwd *pwent,
 		} else {
 			close (1);
 			close (2);
-			open ("/dev/null", O_RDWR); /* open stdout - fd 1 */
-			open ("/dev/null", O_RDWR); /* open stderr - fd 2 */
+			gdm_open_dev_null (O_RDWR); /* open stdout - fd 1 */
+			gdm_open_dev_null (O_RDWR); /* open stderr - fd 2 */
 			gdm_error (_("%s: Could not open ~/.xsession-errors"),
 				   "run_session_child");
 		}
 	} else {
 		close (1);
 		close (2);
-		open ("/dev/null", O_RDWR); /* open stdout - fd 1 */
-		open ("/dev/null", O_RDWR); /* open stderr - fd 2 */
+		gdm_open_dev_null (O_RDWR); /* open stdout - fd 1 */
+		gdm_open_dev_null (O_RDWR); /* open stderr - fd 2 */
 	}
 
 	close (0);
-	open ("/dev/null", O_RDONLY); /* open stdin - fd 0 */
+	gdm_open_dev_null (O_RDONLY); /* open stdin - fd 0 */
 
 	/* Run the PreSession script */
 	if (gdm_slave_exec_script (d, GdmPreSession,
 				   login, pwent,
-				   TRUE /* pass_stdout */) != EXIT_SUCCESS &&
+				   TRUE /* pass_stdout */,
+				   TRUE /* set_parent */) != EXIT_SUCCESS &&
 	    /* ignore errors in failsafe modes */
 	    ! failsafe) 
 		/* If script fails reset X server and restart greeter */
@@ -2840,7 +2970,8 @@ gdm_slave_session_stop (pid_t sesspid)
 	    /* Execute post session script */
 	    gdm_debug ("gdm_slave_session_cleanup: Running post session script");
 	    gdm_slave_exec_script (d, GdmPostSession, local_login, pwent,
-				   FALSE /* pass_stdout */);
+				   FALSE /* pass_stdout */,
+				   TRUE /* set_parent */);
 
 	    gnome_unsetenv ("X_SERVERS");
 	    if (d->type == TYPE_XDMCP)
@@ -2934,6 +3065,7 @@ gdm_slave_term_handler (int sig)
 
 	gdm_server_stop (d);
 	gdm_verify_cleanup (d);
+
 	_exit (DISPLAY_ABORT);
 }
 
@@ -3003,6 +3135,8 @@ gdm_slave_child_handler (int sig)
 	if (pid == d->greetpid && greet) {
 		if (WIFEXITED (status) &&
 		    WEXITSTATUS (status) == DISPLAY_RESTARTGREETER) {
+			gdm_slave_desensitize_config ();
+
 			greet = FALSE;
 			d->greetpid = 0;
 			whack_greeter_fds ();
@@ -3355,33 +3489,109 @@ gdm_child_exit (gint status, const gchar *format, ...)
     _exit (status);
 }
 
+void
+gdm_slave_whack_temp_auth_file (void)
+{
+	uid_t old;
+
+	old = geteuid ();
+	if (old != 0)
+		seteuid (0);
+	if (d->xnest_temp_auth_file != NULL)
+		unlink (d->xnest_temp_auth_file);
+	g_free (d->xnest_temp_auth_file);
+	d->xnest_temp_auth_file = NULL;
+	if (old != 0)
+		seteuid (old);
+}
+
+static void
+create_temp_auth_file (void)
+{
+	if (d->type == TYPE_FLEXI_XNEST &&
+	    d->xnest_auth_file != NULL) {
+		if (d->xnest_temp_auth_file != NULL)
+			unlink (d->xnest_temp_auth_file);
+		g_free (d->xnest_temp_auth_file);
+		d->xnest_temp_auth_file =
+			copy_auth_file (d->server_uid,
+					GdmUserId,
+					d->xnest_auth_file);
+	}
+}
+
+static void
+set_xnest_parent_stuff (void)
+{
+	if (d->type == TYPE_FLEXI_XNEST) {
+		gnome_setenv ("GDM_PARENT_DISPLAY", d->xnest_disp, TRUE);
+		if (d->xnest_temp_auth_file != NULL) {
+			gnome_setenv ("GDM_PARENT_XAUTHORITY",
+				      d->xnest_temp_auth_file, TRUE);
+			g_free (d->xnest_temp_auth_file);
+			d->xnest_temp_auth_file = NULL;
+		}
+	}
+}
+
 static gint
 gdm_slave_exec_script (GdmDisplay *d, const gchar *dir, const char *login,
-		       struct passwd *pwent, gboolean pass_stdout)
+		       struct passwd *pwent, gboolean pass_stdout,
+		       gboolean set_parent)
 {
     pid_t pid;
-    gchar *script, *defscript;
-    const char *scr;
+    char *script;
     gchar **argv;
     gint status;
     char *x_servers_file;
 
-    if (!d || !dir)
+    if (!d || ve_string_empty (dir))
 	return EXIT_SUCCESS;
 
     script = g_strconcat (dir, "/", d->name, NULL);
-    defscript = g_strconcat (dir, "/Default", NULL);
-
-    if (access (script, R_OK|X_OK) == 0) {
-	    scr = script;
-    } else if (access (defscript, R_OK|X_OK) == 0)  {
-	    scr = defscript;
-    } else {
+    if (access (script, R_OK|X_OK) != 0) {
 	    g_free (script);
-	    g_free (defscript);
+	    script = NULL;
+    }
+    if (script == NULL &&
+	! ve_string_empty (d->hostname)) {
+	    script = g_strconcat (dir, "/", d->hostname, NULL);
+	    if (access (script, R_OK|X_OK) != 0) {
+		    g_free (script);
+		    script = NULL;
+	    }
+    }
+    if (script == NULL &&
+	d->type == TYPE_XDMCP) {
+	    script = g_strconcat (dir, "/XDMCP", NULL);
+	    if (access (script, R_OK|X_OK) != 0) {
+		    g_free (script);
+		    script = NULL;
+	    }
+    }
+    if (script == NULL &&
+	SERVER_IS_FLEXI (d)) {
+	    script = g_strconcat (dir, "/Flexi", NULL);
+	    if (access (script, R_OK|X_OK) != 0) {
+		    g_free (script);
+		    script = NULL;
+	    }
+    }
+    if (script == NULL) {
+	    script = g_strconcat (dir, "/Default", NULL);
+	    if (access (script, R_OK|X_OK) != 0) {
+		    g_free (script);
+		    script = NULL;
+	    }
+    }
+    
+    if (script == NULL) {
 	    return EXIT_SUCCESS;
     }
 
+    if (set_parent)
+	    create_temp_auth_file ();
+
     pid = gdm_fork_extra ();
 
     switch (pid) {
@@ -3390,15 +3600,15 @@ gdm_slave_exec_script (GdmDisplay *d, const gchar *dir, const char *login,
         closelog ();
 
 	close (0);
-	open ("/dev/null", O_RDONLY); /* open stdin - fd 0 */
+	gdm_open_dev_null (O_RDONLY); /* open stdin - fd 0 */
 
 	if ( ! pass_stdout) {
 		close (1);
 		close (2);
 		/* No error checking here - if it's messed the best response
 		 * is to ignore & try to continue */
-		open ("/dev/null", O_RDWR); /* open stdout - fd 1 */
-		open ("/dev/null", O_RDWR); /* open stderr - fd 2 */
+		gdm_open_dev_null (O_RDWR); /* open stdout - fd 1 */
+		gdm_open_dev_null (O_RDWR); /* open stderr - fd 2 */
 	}
 
 	gdm_close_all_descriptors (3 /* from */, -1 /* except */);
@@ -3425,6 +3635,9 @@ gdm_slave_exec_script (GdmDisplay *d, const gchar *dir, const char *login,
 	        gnome_setenv ("SHELL", "/bin/sh", TRUE);
         }
 
+	if (set_parent)
+		set_xnest_parent_stuff ();
+
 	/* some env for use with the Pre and Post scripts */
 	x_servers_file = g_strconcat (GdmServAuthDir,
 				      "/", d->name, ".Xservers", NULL);
@@ -3439,22 +3652,26 @@ gdm_slave_exec_script (GdmDisplay *d, const gchar *dir, const char *login,
 	gnome_setenv ("PATH", GdmRootPath, TRUE);
 	gnome_setenv ("RUNNING_UNDER_GDM", "true", TRUE);
 	gnome_unsetenv ("MAIL");
-	argv = ve_split (scr);
+	argv = ve_split (script);
 	execv (argv[0], argv);
-	syslog (LOG_ERR, _("gdm_slave_exec_script: Failed starting: %s"), scr);
+	syslog (LOG_ERR, _("gdm_slave_exec_script: Failed starting: %s"),
+		script);
 	_exit (EXIT_SUCCESS);
 	    
     case -1:
+	if (set_parent)
+		gdm_slave_whack_temp_auth_file ();
 	g_free (script);
-	g_free (defscript);
 	syslog (LOG_ERR, _("gdm_slave_exec_script: Can't fork script process!"));
 	return EXIT_SUCCESS;
 	
     default:
 	gdm_wait_for_extra (&status);
 
+	if (set_parent)
+		gdm_slave_whack_temp_auth_file ();
+
 	g_free (script);
-	g_free (defscript);
 
 	if (WIFEXITED (status))
 	    return WEXITSTATUS (status);
@@ -3686,4 +3903,16 @@ gdm_slave_handle_notify (const char *msg)
 	}
 }
 
+/* do cleanup but only if we are a slave, if we're not a slave, just
+ * return FALSE */
+gboolean
+gdm_slave_final_cleanup (void)
+{
+	if (getpid () != d->slavepid)
+		return FALSE;
+	gdm_debug ("slave killing self");
+	gdm_slave_term_handler (SIGTERM);
+	return TRUE;
+}
+
 /* EOF */
