@@ -77,6 +77,7 @@ gdm_verify_user (GdmDisplay *d,
 		    g_free (login);
 		    return NULL;
 	    }
+	    gdm_slave_greeter_ctl_no_ret (GDM_MSG, "");
     } else {
 	    login = g_strdup (username);
     }
@@ -87,7 +88,8 @@ gdm_verify_user (GdmDisplay *d,
     
     /* Request the user's password */
     if (pwent != NULL &&
-        ve_string_empty (ppasswd)) {
+        (ve_string_empty (ppasswd) ||
+	 (local && gdm_is_a_no_password_user (login)))) {
 	    /* eeek a passwordless account */
 	    passwd = g_strdup ("");
     } else {
